﻿namespace PropertiesVsFieldsSyntax {

	class A {
		public int fa;
		public readonly int fb;

		public int pa { get; set; }
		public int pb { get; }

		public A() {
			fa = -1;
			fb = -2;

			pa = -1;
			pb = -2;
		}
	}

	class B {
		public int pa { get; set; }
		private int pb { get; set; }
		public int pc { get; private set; }
		public int pd { private get; set; }
	}

	class Program {
		static void Main(string[] args) {

			var a1 = new A();
			a1.fa = 10;
			// Error: A readonly field cannot be assigned to(except in a constructor or init-only setter of the type in which the field is defined or a variable initializer)
			// a1.fb = 20;	// <- readonly field -> readonly int

			a1.pa = 10;
			// Error: Property or indexer 'A.pb' cannot be assigned to -- it is read only
			// a1.pb = 20;	// <- read-only property -> { get; }

			// ------------------------- //

			var x1 = new X();
			x1.fa = 10;
			// Error: A readonly field cannot be assigned to(except in a constructor or init-only setter of the type in which the field is defined or a variable initializer)
			// x1.fb = 20;	// <- readonly field -> readonly int

			x1.pa = 10;
			// Error: Property or indexer 'X.pb' cannot be assigned to --it is read only
			// x1.pb = 20;	// <- read-only property -> { get; }

			// Error: Init-only property or indexer 'X.pc' can only be assigned in an object initializer, or on 'this' or 'base' in an instance constructor or an 'init' accessor.
			// x1.pc = 30;	// <- property with init-only setter -> { get; init; }

			Console.WriteLine($"x1: .fa = {x1.fa}, .fb = {x1.fb}, .pa = {x1.pa}, .pb = {x1.pb}, .pc = {x1.pc}");

			// ------------------------- //

			var x2 = new X(10, 20, 10, 20, 30);

			Console.WriteLine($"x2: .fa = {x2.fa}, .fb = {x2.fb}, .pa = {x2.pa}, .pb = {x2.pb}, .pc = {x2.pc}");

			// ------------------------- //

			// Error: A readonly field cannot be assigned to(except in a constructor or init-only setter of the type in which the field is defined or a variable initializer)
			// var x3 = new X() { fb = 20 };	// <- readonly field -> readonly int

			var x4 = new X() { fa = 10, pa = 10, pc = 30 };

			Console.WriteLine($"x4: .fa = {x4.fa}, .fb = {x4.fb}, .pa = {x4.pa}, .pb = {x4.pb}, .pc = {x4.pc}");
		}
	}

	class X {
		public int fa;
		public readonly int fb;

		public int pa { get; set; }
		public int pb { get; }
		public int pc { get; init; }

		public X() {
			fa = -1;
			fb = -2;

			pa = -1;
			pb = -2;
			pc = -3;
		}

		public X(int fa, int fb, int pa, int pb, int pc) {
			this.fa = fa;
			this.fb = fb;
			this.pa = pa;
			this.pb = pb;
			this.pc = pc;
		}
	}
}